@@ -5,7 +5,7 @@ import sysconfig
 from importlib.util import cache_from_source
 from typing import Any, Callable, Dict, Generator, Iterable, List, Optional, Set, Tuple
 
-from pipenv.patched.pip._internal.exceptions import UninstallationError
+from pipenv.patched.pip._internal.exceptions import LegacyDistutilsInstall, UninstallMissingRecord
 from pipenv.patched.pip._internal.locations import get_bin_prefix, get_bin_user
 from pipenv.patched.pip._internal.metadata import BaseDistribution
 from pipenv.patched.pip._internal.utils.compat import WINDOWS
@@ -38,7 +38,7 @@ def _script_names(
 
 
 def _unique(
-    fn: Callable[..., Generator[Any, None, None]]
+    fn: Callable[..., Generator[Any, None, None]],
 ) -> Callable[..., Generator[Any, None, None]]:
     @functools.wraps(fn)
     def unique(*args: Any, **kw: Any) -> Generator[Any, None, None]:
@@ -61,7 +61,7 @@ def uninstallation_paths(dist: BaseDistribution) -> Generator[str, None, None]:
 
     UninstallPathSet.add() takes care of the __pycache__ .py[co].
 
-    If RECORD is not found, raises UninstallationError,
+    If RECORD is not found, raises an error,
     with possible information from the INSTALLER file.
 
     https://packaging.python.org/specifications/recording-installed-packages/
@@ -71,17 +71,7 @@ def uninstallation_paths(dist: BaseDistribution) -> Generator[str, None, None]:
 
     entries = dist.iter_declared_entries()
     if entries is None:
-        msg = "Cannot uninstall {dist}, RECORD file not found.".format(dist=dist)
-        installer = dist.installer
-        if not installer or installer == "pip":
-            dep = "{}=={}".format(dist.raw_name, dist.version)
-            msg += (
-                " You might be able to recover from this via: "
-                "'pip install --force-reinstall --no-deps {}'.".format(dep)
-            )
-        else:
-            msg += " Hint: The package was installed by {}.".format(installer)
-        raise UninstallationError(msg)
+        raise UninstallMissingRecord(distribution=dist)
 
     for entry in entries:
         path = os.path.join(location, entry)
@@ -172,8 +162,7 @@ def compress_for_output_listing(paths: Iterable[str]) -> Tuple[Set[str], Set[str
             folders.add(os.path.dirname(path))
         files.add(path)
 
-    # probably this one https://github.com/python/mypy/issues/390
-    _normcased_files = set(map(os.path.normcase, files))  # type: ignore
+    _normcased_files = set(map(os.path.normcase, files))
 
     folders = compact(folders)
 
@@ -274,7 +263,7 @@ class StashedUninstallPathSet:
 
     def commit(self) -> None:
         """Commits the uninstall by removing stashed files."""
-        for _, save_dir in self._save_dirs.items():
+        for save_dir in self._save_dirs.values():
             save_dir.cleanup()
         self._moves = []
         self._save_dirs = {}
@@ -316,7 +305,7 @@ class UninstallPathSet:
         # Create local cache of normalize_path results. Creating an UninstallPathSet
         # can result in hundreds/thousands of redundant calls to normalize_path with
         # the same args, which hurts performance.
-        self._normalize_path_cached = functools.lru_cache()(normalize_path)
+        self._normalize_path_cached = functools.lru_cache(normalize_path)
 
     def _permitted(self, path: str) -> bool:
         """
@@ -368,7 +357,7 @@ class UninstallPathSet:
             )
             return
 
-        dist_name_version = f"{self._dist.raw_name}-{self._dist.version}"
+        dist_name_version = f"{self._dist.raw_name}-{self._dist.raw_version}"
         logger.info("Uninstalling %s:", dist_name_version)
 
         with indent_log():
@@ -510,22 +499,19 @@ class UninstallPathSet:
                     paths_to_remove.add(f"{path}.pyo")
 
         elif dist.installed_by_distutils:
-            raise UninstallationError(
-                "Cannot uninstall {!r}. It is a distutils installed project "
-                "and thus we cannot accurately determine which files belong "
-                "to it which would lead to only a partial uninstall.".format(
-                    dist.raw_name,
-                )
-            )
+            raise LegacyDistutilsInstall(distribution=dist)
 
         elif dist.installed_as_egg:
             # package installed by easy_install
             # We cannot match on dist.egg_name because it can slightly vary
             # i.e. setuptools-0.6c11-py2.6.egg vs setuptools-0.6rc11-py2.6.egg
-            paths_to_remove.add(dist_location)
-            easy_install_egg = os.path.split(dist_location)[1]
+            # XXX We use normalized_dist_location because dist_location my contain
+            # a trailing / if the distribution is a zipped egg
+            # (which is not a directory).
+            paths_to_remove.add(normalized_dist_location)
+            easy_install_egg = os.path.split(normalized_dist_location)[1]
             easy_install_pth = os.path.join(
-                os.path.dirname(dist_location),
+                os.path.dirname(normalized_dist_location),
                 "easy-install.pth",
             )
             paths_to_remove.add_pth(easy_install_pth, "./" + easy_install_egg)
